/*! jwt-auth-react v3.2.9-0 | Apache-2.0 Copyright (2020) Arkadip Bhattacharya */
import React, { useState, useEffect, useContext } from 'react';
import { Route, Redirect } from 'react-router-dom';

/*
 * Copyright 2020. Arkadip Bhattacharya
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
/**
 * Initialized Auth Context
 */
const Auth = React.createContext(null);
const AuthContextProvider = Auth.Provider;
const AuthContext = Auth;

/*
 * Copyright 2020. Arkadip Bhattacharya
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
/**
 * Token Object
 */
class TokenObject {
    constructor(tokenName) {
        this.tokenName = tokenName;
    }
    /**
     * Checks if the token is expired or not
     * @returns {boolean} true: if the token is expired
     */
    isTokenExpire() {
        if (this.isTokenExist()) {
            const _e = JSON.parse(localStorage[this.tokenName])["expiresAt"];
            return _e < new Date();
        }
        else {
            return true;
        }
    }
    /**
     * @returns {boolean} true: if the token exists
     */
    isTokenExist() {
        return localStorage[this.tokenName] !== undefined;
    }
    setToken(value, expiresAt) {
        //{value: xxx, expiresAt: ttt}
        const json = { "value": value, "expiresAt": expiresAt };
        localStorage[this.tokenName] = JSON.stringify(json);
    }
    getToken() {
        if (this.isTokenExist()) {
            if (!this.isTokenExpire()) {
                const _t = JSON.parse(localStorage[this.tokenName]);
                const token = _t["value"];
                const expiresAt = _t["expiresAt"];
                return { authToken: token, expireAt: expiresAt };
            }
            else {
                return { authToken: null, expireAt: null };
            }
        }
        else {
            return { authToken: null, expireAt: null };
        }
    }
    removeToken() {
        localStorage.removeItem(this.tokenName);
    }
}

/*
 * Copyright 2020. Arkadip Bhattacharya
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
/**
 * AuthProvider, Context provider that passes the auth down to the components
 * @param cookieName - Name of the auth Cookie. Default: '_authUser'
 * @param children - React.Component | React.FunctionalComponent
 * @constructor
 */
const AuthProvider = ({ cookieName, children }) => {
    const JWT_token = new TokenObject(cookieName);
    const [authObject, setAuthObject] = useState(JWT_token.getToken());
    useEffect(() => {
        if (authObject.authToken === undefined || authObject.authToken === null || authObject.expireAt === null) {
            JWT_token.removeToken();
        }
        else {
            JWT_token.setToken(authObject.authToken, authObject.expireAt);
        }
    }, [authObject]);
    return (React.createElement(React.Fragment, null,
        React.createElement(AuthContextProvider, { value: { authObject: authObject, setAuthToken: setAuthObject } }, children)));
};
AuthProvider.defaultProps = {
    cookieName: `_authUser`
};

/*
 * Copyright 2020. Arkadip Bhattacharya
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
/**
 * Checks if the USER is logged in or not
 * @returns {{auth: boolean, authToken: string}} AuthObject
 */
const isAuth = () => {
    const value = useContext(AuthContext);
    if ((value === null || value === void 0 ? void 0 : value.authObject.authToken) && (value === null || value === void 0 ? void 0 : value.authObject.expireAt)) {
        if (value.authObject.expireAt < new Date()) {
            return { auth: true, authToken: value.authObject.authToken };
        }
        else {
            value.setAuthToken({ authToken: null, expireAt: null });
            return { auth: false, authToken: null };
        }
    }
    else {
        return { auth: false, authToken: null };
    }
};

/*
 * Copyright 2020. Arkadip Bhattacharya
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
/**
 *
 * @param component - Component to
 * @param loginPath
 * @param path
 * @param rest
 * @constructor
 */
const PrivateRoute = ({ component, loginPath, path }) => {
    const { auth } = isAuth();
    if (auth) {
        return React.createElement(Route, { component: component, path: path });
    }
    else {
        return React.createElement(Redirect, { to: loginPath });
    }
};

/*
 * Copyright 2020. Arkadip Bhattacharya
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
/**
 * Auth Login function
 * @param token - Token from the backend server
 * @param expiresIn - Time takes to expires a token in minutes
 */
const loginAuth = (token, expiresIn) => {
    const expTime = new Date(new Date().getTime() + (expiresIn * 60 * 1000));
    try {
        const c = useContext(AuthContext);
        c === null || c === void 0 ? void 0 : c.setAuthToken({ authToken: token, expireAt: expTime });
        return true;
    }
    catch (e) {
        return false;
    }
};
/**
 * Auth Logout Function
 */
const logoutAuth = () => {
    try {
        const c = useContext(AuthContext);
        if (c === null || c === void 0 ? void 0 : c.authObject.authToken) {
            c.setAuthToken({ authToken: null, expireAt: null });
            return true;
        }
        else {
            return false;
        }
    }
    catch (e) {
        return false;
    }
};

/*
 * Copyright 2020. Arkadip Bhattacharya
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
const authHeader = () => {
    const c = useContext(AuthContext);
    if (c === null || c === void 0 ? void 0 : c.authObject.authToken) {
        return `Bearer ${c.authObject.authToken}`;
    }
    else {
        return null;
    }
};

export { AuthProvider, PrivateRoute, authHeader, isAuth, loginAuth, logoutAuth };
